package main

type API struct {
	BasePath             string                `yaml:"basepath,omitempty"`
	Host                 string                `yaml:"host,omitempty"`
	Types                map[string]*APIType   `yaml:"types,omitempty"`
	Methods              map[string]*APIMethod `yaml:"methods,omitempty"`
	Namespace            string                `yaml:"namespace"`
	Imports              map[string]string     `yaml:"imports"`
	UsedImportsTypes     map[string]bool       `yaml:"used_imports_types"`
	UsedImportsFunctions map[string]bool       `yaml:"used_imports_functions"`
}
type APIFieldTag struct {
	Key  string
	Name string
	Opts []string
}
type APIField struct {
	Type   string                 `yaml:"type,omitempty"`
	Array  bool                   `yaml:"array,omitempty"`
	Desc   string                 `yaml:"desc,omitempty"`
	Map    bool                   `yaml:"map,omitempty"`
	Mapkey string                 `yaml:"mapkey,omitempty"`
	Mapval string                 `yaml:"mapval,omitempty"`
	Tags   map[string]APIFieldTag `yaml:"tags,omitempty"`
}

func (a *APIField) String() string {
	if a.Array {
		return "[]" + a.Type
	} else {
		return a.Type
	}
}

type APIType struct {
	Name   string               `yaml:"name,omitempty"`
	Desc   string               `yaml:"desc,omitempty"`
	Fields map[string]*APIField `yaml:"fields,omitempty"`
	Col    string               `yaml:"col,omitempty"`
}

type APIParamType struct {
	Typename  string
	Ispointer bool
	IsArray   bool
}

type APIMethod struct {
	Desc    string `yaml:"desc"`
	Verb    string `yaml:"verb"`
	Path    string `yaml:"path"`
	Perm    string `yaml:perm`
	ReqType *APIParamType
	ResType *APIParamType
	Raw     bool `yaml:"raw"`
}

func APIParamTypeToString(t *APIParamType) string {
	ret := ""
	if t.IsArray {
		ret = "[]"
		if t.Ispointer {
			ret = ret + "*"
		}
		ret = ret + t.Typename
		return ret
	}
	if t.Ispointer {
		ret = ret + "*"
	}
	ret = ret + t.Typename
	return ret
}

func APIParamTypeDecToString(t *APIParamType) string {
	ret := ""
	if t.IsArray {
		ret = "[]"
		if t.Ispointer {
			ret = ret + "*"
		}
		ret = ret + t.Typename
		return ret
	}
	if t.Ispointer {
		ret = ret + "*"
	}
	ret = ret + t.Typename
	return ret
}

func APIParamTypeUseRef(t *APIParamType) string {
	if t.IsArray || !t.Ispointer {
		return "&"
	}
	return ""
}
